from django.contrib.contenttypes.models import ContentType
from django.core.validators import MinValueValidator
from django.forms import (
    CharField,
    FloatField, BooleanField
)
from django.urls import reverse_lazy

from dcim.models import Platform
from netbox.forms import (
    NetBoxModelForm,
)
from netbox_storage.choices import DeviceTypeChoices
from netbox_storage.models import Drive, Filesystem, Partition, PhysicalVolume, VolumeGroup, LogicalVolume, \
    MountedVolume, LinuxDevice, TemplateConfigurationDrive
from utilities.forms import (
    DynamicModelChoiceField, APISelect,
)
from virtualization.models import Cluster, ClusterType, VirtualMachine


class PartitionTemplateForm(NetBoxModelForm):
    size = FloatField(
        label="Size (GB)",
        help_text="The size of the logical volume e.g. 25",
        validators=[MinValueValidator(0.1)],
    )
    drive = DynamicModelChoiceField(
        queryset=Drive.objects.all(),
        help_text="The Cluster Type of the drive",
    )
    platform = DynamicModelChoiceField(
        queryset=Platform.objects.all(),
        help_text="Mapping between Volume and platform e.g. Rocky Linux 8",
    )
    mountpoint = CharField(
        required=False,
        label="Mountpoint",
        help_text="The mounted point of the volume e.g. /var/lib/docker",
    )
    fs_type = DynamicModelChoiceField(
        required=False,
        queryset=Filesystem.objects.all(),
        label="Filesystem Name",
        widget=APISelect(
            attrs={"data-url": reverse_lazy("plugins-api:netbox_storage-api:filesystem-list")}
        ),
        help_text="The Filesystem of the Volume e.g. ext4",
    )
    fs_options = CharField(
        required=False,
        label="FS Options",
        help_text="The mounted point of the volume e.g. /var/lib/docker",
        initial='defaults',
    )
    label = CharField(
        required=False,
        label="Partition Label",
        help_text="The mounted point of the volume e.g. /var/lib/docker",
    )

    fieldsets = (
        (
            "Template for Platform",
            (
                "platform",
            ),
        ),
        (
            "Select drive",
            (
                "drive",
            ),
        ),
        (
            "Size of new Partition",
            (
                "size",
            ),
        ),
        (
            "Create Filesystem and mount it",
            (
                "mountpoint",
                "fs_type",
                "fs_options",
                "label",
            ),
        ),
    )

    class Meta:
        model = LinuxDevice
        fields = [
            "size"
        ]

    def save(self, *args, **kwargs):
        # Drive Object
        drive = Drive.objects.get(pk=self.cleaned_data['drive'].id)

        # Linux Device & Drive Type ID
        linux_device_type_pk = ContentType.objects.get(app_label='netbox_storage', model='linuxdevice').pk
        drive_type_pk = ContentType.objects.get(app_label='netbox_storage', model='drive').pk

        # The linux device drive object
        linux_device_drive = LinuxDevice.objects.get(object_id=drive.pk, content_type_id=drive_type_pk)

        # All linux devices, which are partition and have the drive as a parent object
        linux_device_partitions = LinuxDevice.objects.filter(content_type_id=linux_device_type_pk,
                                                             object_id=linux_device_drive.pk,
                                                             type=DeviceTypeChoices.CHOICE_PARTITION)

        # The new device name e.g. /dev/sda + 1 -> partition count
        new_partition_count = linux_device_partitions.count() + 1
        device_name_prefix = drive.device_name()
        device_name = device_name_prefix + str(new_partition_count)

        # Add the value to the Linux Device Partition Object
        self.instance.device = device_name
        self.instance.type = DeviceTypeChoices.CHOICE_PARTITION
        self.instance.object_id = linux_device_drive.pk
        self.instance.content_type_id = linux_device_type_pk
        linux_device_partition = super().save(*args, **kwargs)

        # Mount the Partition if the values are set
        if self.cleaned_data['mountpoint'] and self.cleaned_data['fs_type'] and self.cleaned_data['fs_options']:
            MountedVolume.objects.create(linux_device=linux_device_partition,
                                         mount_point=self.cleaned_data['mountpoint'],
                                         fs_type=self.cleaned_data['fs_type'],
                                         options=self.cleaned_data['fs_options'])
        return linux_device_partition


class VolumeSimpleForm(NetBoxModelForm):
    """Form for creating a new Drive object."""
    # ct = ClusterType.objects.filter(name="Storage").values_list('id', flat=True)[0]
    size = FloatField(
        label="Size (GB)",
        help_text="The size of the logical volume e.g. 25",
        validators=[MinValueValidator(0.1)],
        required=False
    )
    lv_name = CharField(
        label="LV Name",
        help_text="The logical volume name e.g. lv_data",
        required=False
    )
    vg_name = CharField(
        label="VG Name",
        help_text="The volume group name e.g. vg_data",
        required=False
    )
    mountpoint = CharField(
        label="Mountpoint",
        help_text="The mounted point of the volume e.g. /var/lib/docker",
        required=False
    )
    hard_drive_label = CharField(
        label="Hard Drive Label",
        help_text="The label of the hard drive e.g. D",
        required=False
    )
    fs = DynamicModelChoiceField(
        queryset=Filesystem.objects.all(),
        label="Filesystem Name",
        widget=APISelect(
            attrs={"data-url": reverse_lazy("plugins-api:netbox_storage-api:filesystem-list")}
        ),
        help_text="The Filesystem of the Volume e.g. ext4",
        required=False
    )
    cluster_type = DynamicModelChoiceField(
        queryset=ClusterType.objects.all(),
        help_text="The Cluster Type of the drive",
    )
    cluster = DynamicModelChoiceField(
        queryset=Cluster.objects.all(),
        query_params={
            'type_id': '$cluster_type'  # ClusterType.objects.filter(name="Storage").values_list('id', flat=True)[0]
        },
        help_text="The Storage Cluster of the drive",
    )
    description = CharField(
        required=False,
        label="Description",
        help_text="Short Description e.g. Hard Drive 1 on SSD Cluster",
    )

    class Meta:
        model = Drive

        fields = (
            "size",
            "cluster",
            "description",
        )

    def save(self, *args, **kwargs):
        drive = super().save(*args, **kwargs)
        return drive



from django.db.models import Q

from netbox.filtersets import NetBoxModelFilterSet

from netbox_storage.models import TemplateConfigurationDrive


class TemplateConfigurationDriveFilter(NetBoxModelFilterSet):
    class Meta:
        model = TemplateConfigurationDrive
        fields = ["platform", "drive"]

    def search(self, queryset, name, value):
        """Perform the filtered search."""
        if not value.strip():
            return queryset



from django.contrib.contenttypes.models import ContentType
from django.core.validators import MinValueValidator
from django.forms import CharField, FloatField, BooleanField
from django.urls import reverse_lazy

from dcim.models import Platform
from netbox.forms import NetBoxModelForm
from netbox_storage.models import Drive, Filesystem, LinuxDevice, VolumeGroup, PhysicalVolume, MountedVolume, \
    LogicalVolume
from utilities.forms import DynamicModelChoiceField, APISelect
from virtualization.models import ClusterType, Cluster, VirtualMachine


class LVMTemplateForm(NetBoxModelForm):
    platform = DynamicModelChoiceField(
        required=False,
        queryset=Platform.objects.all(),
        help_text="Mapping between drive and platform  e.g. Rocky Linux 9",
    )
    drive = DynamicModelChoiceField(
        queryset=Drive.objects.all(),
        help_text="The Cluster Type of the drive",
    )

    cluster_type = DynamicModelChoiceField(
        queryset=ClusterType.objects.all(),
        help_text="The Cluster Type of the drive",
    )
    cluster = DynamicModelChoiceField(
        queryset=Cluster.objects.all(),
        query_params={
            'type_id': '$cluster_type'  # ClusterType.objects.filter(name="Storage").values_list('id', flat=True)[0]
        },
        help_text="The Storage Cluster of the drive",
    )

    partition = DynamicModelChoiceField(
        queryset=LinuxDevice.objects.filter(pk=1),
        label="Partition",
        help_text="The Storage Cluster of the drive",
        required=False
    )

    lv_name = CharField(
        label="LV Name",
        help_text="Logical Volume Name e.g. lv_docker",
    )
    vg_name = CharField(
        label="VG Name",
        help_text="Volume Group Name e.g. vg_docker",
    )
    size = FloatField(
        label="Size (GB)",
        help_text="The size of the logical volume e.g. 25",
        validators=[MinValueValidator(0.1)],
    )
    mountpoint = CharField(
        label="Mountpoint",
        help_text="The mounted point of the volume e.g. /var/lib/docker",
    )
    fs_type = DynamicModelChoiceField(
        queryset=Filesystem.objects.all(),
        label="Filesystem Name",
        widget=APISelect(
            attrs={"data-url": reverse_lazy("plugins-api:netbox_storage-api:filesystem-list")}
        ),
        help_text="The Filesystem of the Volume e.g. ext4",
    )
    fs_options = CharField(
        label="Filesystem Options",
        help_text="The mounted point of the volume e.g. /var/lib/docker",
        initial='defaults',
    )
    checkbox_partition = BooleanField(
        label="Create extra partition",
        help_text="Create an extra partition e.g. /dev/sdb1",
        required=False
    )

    virtual_machine = DynamicModelChoiceField(
        required=False,
        label="Virtual Machine",
        queryset=VirtualMachine.objects.all(),
        help_text="Mapping between drive and virtual machine  e.g. vm-testinstall-01",
    )

    fieldsets = (
        (
            "Template for Platform",
            (
                "platform",
            ),
        ),
        (
            "Drive & Partition Config",
            (
                "drive",
                "checkbox_partition"
            ),
        ),
        (
            "LVM Configuration",
            (
                "lv_name",
                "vg_name",
                "size",
            ),
        ),
        (
            "Create Filesystem and mount it",
            (
                "mountpoint",
                "fs_type",
                "fs_options",
            ),
        ),
    )

    class Meta:
        model = LogicalVolume
        fields = [
            "lv_name",
            "size",
        ]

    def save(self, *args, **kwargs):
        print(f"Drive: {self.cleaned_data['drive']}")
        print(f"Platform: {self.cleaned_data['platform']}")
        print(f"LV Name: {self.cleaned_data['lv_name']}")
        print(f"VG Name: {self.cleaned_data['vg_name']}")
        print(f"Size: {self.cleaned_data['size']}")
        print(f"Mountpoint: {self.cleaned_data['mountpoint']}")
        print(f"Filesystem: {self.cleaned_data['fs_type']}")
        print(f"Filesystem Options: {self.cleaned_data['fs_options']}")
        print(f"Checkbox Partition: {self.cleaned_data['checkbox_partition']}")

        device_name_prefix = Drive.objects.get(pk=self.cleaned_data['drive'].id).device_name()
        print(f"Device Name: {device_name_prefix}")

        linux_parent_device = LinuxDevice.objects.get(device=device_name_prefix)
        if self.cleaned_data['checkbox_partition']:
            device_name = device_name_prefix + str(1)
            linuxdevice_type_pk = ContentType.objects.get(app_label='netbox_storage', model='linuxdevice').pk
            pv_linux_device = LinuxDevice.objects.create(device=device_name,
                                                         type='Partition',
                                                         size=linux_parent_device.size,
                                                         object_id=linux_parent_device.pk,
                                                         content_type_id=linuxdevice_type_pk)
        else:
            pv_linux_device = linux_parent_device

        volume_group = VolumeGroup.objects.create(vg_name=self.cleaned_data['vg_name'])

        pv = PhysicalVolume.objects.create(linux_device=pv_linux_device, vg=volume_group)

        lv_type_pk = ContentType.objects.get(app_label='netbox_storage', model='logicalvolume').pk

        self.instance.vg = volume_group
        lv = super().save(*args, **kwargs)

        linux_device_mapper = LinuxDevice.objects.create(device='/dev/mapper/vg_system-lv_home',
                                                         type='LVM',
                                                         size=pv_linux_device.size,
                                                         object_id=lv.pk,
                                                         content_type_id=lv_type_pk)

        MountedVolume.objects.create(linux_device=linux_device_mapper,
                                     mount_point=self.cleaned_data['mountpoint'],
                                     fs_type=self.cleaned_data['fs_type'],
                                     options=self.cleaned_data['fs_options'])
        return lv




import django_filters
from django.forms import (
    CharField, FloatField, HiddenInput
)

from netbox.forms import (
    NetBoxModelBulkEditForm,
    NetBoxModelFilterSetForm,
    NetBoxModelImportForm,
    NetBoxModelForm,
)
from netbox_storage.choices import DeviceTypeChoices

from netbox_storage.models import LinuxDevice
from utilities.forms import DynamicModelChoiceField, ChoiceField, add_blank_choice, StaticSelect


class LinuxDeviceForm(NetBoxModelForm):
    device = CharField(
        label="Device Name",
        help_text="The mounted path of the volume e.g. /var/lib/docker",
    )
    type = ChoiceField(
        choices=add_blank_choice(DeviceTypeChoices),
        label="Device Type",
        help_text="The type of the device e.g. Disk, Partition, LVM",
        initial='',
        widget=StaticSelect()
    )
    size = FloatField(
        label="Size",
        help_text="The size of the device e.g. 200",
    )

    class Meta:
        model = LinuxDevice

        fields = (
            'content_type',
            'object_id',
            "device",
            "type",
            "size",
        )
        widgets = {
            'content_type': HiddenInput(),
            'object_id': HiddenInput(),
        }

"""
    def save(self, *args, **kwargs):
        object_pk = self.cleaned_data['parent_object'].pk

        self.instance.content_type_pk = ContentType.objects.get(app_label='netbox_storage', model='linuxdevice').pk
        self.instance.object_id = object_pk

        linux_device = super().save(*args, **kwargs)

        return linux_device
"""


class LinuxDeviceFilterForm(NetBoxModelFilterSetForm):

    model = LinuxDevice

    device = CharField(
        label="Device",
        help_text="The mounted path of the volume e.g. /var/lib/docker",
    )


class LinuxDeviceImportForm(NetBoxModelImportForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    class Meta:
        model = LinuxDevice

        fields = (
            "device",
            "type",
        )


class LinuxDeviceBulkEditForm(NetBoxModelBulkEditForm):
    model = LinuxDevice

    device = CharField(
        required=False,
        label="Device",
    )
    type = CharField(
        required=False,
        label="Type",
    )

    fieldsets = (
        (
            None,
            ("device", "type"),
        ),
    )



import django_tables2 as tables

from netbox.tables import (
    NetBoxTable,
    ToggleColumn,
)

from netbox_storage.models import Drive


class DriveTable(NetBoxTable):
    """Table for displaying Drives objects."""

    pk = ToggleColumn()
    identifier = tables.Column(
        linkify=True
    )
    size = tables.Column(
        linkify=True
    )
    cluster = tables.Column(
        linkify=True
    )
    object = tables.Column(
        linkify=True,
        verbose_name="Parent Object"
    )

    class Meta(NetBoxTable.Meta):
        model = Drive
        fields = (
            "pk",
            "size",
            "cluster",
            "identifier",
            "description",
        )
        default_columns = (
            "identifier",
            "size",
            "cluster",
        )



from django.contrib.contenttypes.models import ContentType

from extras.plugins import PluginTemplateExtension
from netbox_storage.models import LogicalVolume, StorageConfigurationDrive, \
    TemplateConfigurationDrive, Drive, Partition, LinuxDevice, MountedVolume


class VMTemplateContent(PluginTemplateExtension):
    model = "virtualization.virtualmachine"

    def left_page(self):
        obj = self.context.get("object")

        drives_id = StorageConfigurationDrive.objects.values('drive').filter(virtual_machine=obj)
        partition_dict = {}
        for drive_id in drives_id:
            print(f"Drive ID: {drive_id['drive']}")
            drive = Drive.objects.get(pk=drive_id['drive'])

            if Partition.objects.filter(drive=drive).count() == 0:
                partition_dict[drive] = None
            else:
                partition_dict[drive] = list(Partition.objects.filter(drive=drive))

        platform = obj.platform
        if platform is not None:
            if platform.name.lower().__contains__('windows'):
                return self.render(
                    "netbox_storage/inc/vm_windows_box.html",
                    extra_context={
                        "partition_dict": partition_dict,
                    },
                )
            elif platform.name.lower().__contains__('linux'):
                return self.render(
                    "netbox_storage/inc/linuxvolume_box.html"
                )
        else:
            return self.render(
                "netbox_storage/inc/unknown_os_box.html"
            )


class PlatformTemplateContent(PluginTemplateExtension):
    model = "dcim.platform"

    def right_page(self):
        obj = self.context.get("object")
        parent_object_type = ContentType.objects.get(app_label='dcim', model='platform').pk

        if obj.name.lower().__contains__('windows'):
            drives = Drive.objects.filter(content_type=parent_object_type, object_id=obj.pk)
            partition_dict = {}
            for drive in drives:
                if Partition.objects.filter(drive=drive).count() == 0:
                    partition_dict[drive] = None
                else:
                    partition_dict[drive] = list(Partition.objects.filter(drive=drive))
            return self.render(
                "netbox_storage/inc/template_windows_drive_box.html",
                extra_context={
                    "partition_dict": partition_dict,
                },
            )
        else:
            drives_id = Drive.objects.filter(content_type=parent_object_type, object_id=obj)
            partition_dict = {}
            for drive in drives_id:
                drive_type_id = ContentType.objects.get(app_label='netbox_storage', model='drive').pk
                linux_device_type_id = ContentType.objects.get(app_label='netbox_storage', model='linuxdevice').pk
                # Get Linux Device of Drive e.g. /dev/sda
                linux_device_drive = LinuxDevice.objects.get(content_type_id=drive_type_id,
                                                             object_id=drive.pk,
                                                             type='Disk')

                if LinuxDevice.objects.filter(content_type_id=linux_device_type_id,
                                              object_id=linux_device_drive.pk,
                                              type='Partition').count() == 0:
                    partition_dict[drive] = None
                else:
                    partition_dict[drive] = list(LinuxDevice.objects.filter(content_type_id=linux_device_type_id,
                                                                            object_id=linux_device_drive.pk,
                                                                            type='Partition'))
            return self.render(
                "netbox_storage/inc/template_drive_box.html",
                extra_context={
                    "partition_dict": partition_dict
                }
            )

    def left_page(self):
        obj = self.context.get("object")
        mounted_volumes = MountedVolume.objects.all()
        if obj.name.lower().__contains__('windows'):
            return self.render(
                "netbox_storage/inc/nothing.html",
            )
        else:
            return self.render(
                "netbox_storage/inc/template_volume_box.html",
                extra_context={
                    "mounted_volumes": mounted_volumes
                }
            )


template_extensions = [VMTemplateContent, PlatformTemplateContent]



from .drive import *
from .filesystem import *
from .linux_device import *
from .logical_volume import *
from .mounted_volume import *
from .partition import *
from .physical_volume import *
from .simple import *
from .template import *
from .volume_group import *
